package uhk.salay.npl.demonpl.jpa.model;

import java.io.Serializable;
import java.util.Objects;
import java.util.Set;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import lombok.Data;

@Data
@javax.persistence.Entity
@Table(name = "Entities")
public class Entity implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private int id;
    private String name;
    @OneToOne
    @JoinColumn(name = "type_id", referencedColumnName = "id")
    private Type type;
    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name = "Entities_Files",
            joinColumns = {@JoinColumn(name = "entity_id", referencedColumnName = "id")},
            inverseJoinColumns = {@JoinColumn(name = "file_id", referencedColumnName = "id")}
    )
    private Set<File> files;

    public String getFilesString() {
        StringBuilder stringBuilder = new StringBuilder();
        files.forEach(s -> stringBuilder.append(s.getName()).append(" "));

        return stringBuilder.toString();
    }

    @OneToOne(mappedBy = "entity")
    private Emotion emotion;

    @OneToOne(mappedBy = "entity")
    private Sentiment sentiment;

    public Entity() {
    }


    public Entity(String name, Type type) {
        this.name = name;
        this.type = type;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        Entity entity = (Entity) o;
        return id == entity.id &&
                Objects.equals(name, entity.name) &&
                Objects.equals(type, entity.type);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name, type);
    }
}
