package uhk.salay.npl.demonpl.jpa.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.persistence.*;

import lombok.Data;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

@Data
@javax.persistence.Entity
@Table(name = "Files")
public class File implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private int id;
    private String path;
    @ManyToMany(mappedBy = "files",fetch = FetchType.EAGER)
    private Set<Entity> entities;
    @Column(name = "process_complete")
    private Integer processComplete = 0;
    @Column(name = "parts")
    private Integer part = 0;
    @OneToMany(fetch = FetchType.EAGER)
    @JoinColumn(name = "file_id", referencedColumnName = "id")
    private List<Part> parts = new ArrayList<>();
    @LazyCollection(LazyCollectionOption.FALSE)
    @OneToMany(mappedBy = "file")
    private List<Emotion> emotions = new ArrayList<>();
    @Transient
    private Set<String> categoriesSet = new HashSet<>();
    @Basic
    @Column(name = "categories")
    private String categories;
    @Transient
    private Set<String> conceptSet = new HashSet<>();
    @Basic
    @Column(name = "concept")
    private String concept;
    @Transient
    private Set<String> keywordSet = new HashSet<>();
    @Basic
    @Column(name = "keyword")
    private String keyword;
    @Basic
    @Column(name = "sentiment")
    private Double sentiment = 0.0;

    public File() {
    }

    public File(String path, Set<Entity> entities, Integer processComplete) throws Exception {
        this.path = path;
        this.entities = entities;
        if (processComplete == 0 || processComplete == 1) {
            this.processComplete = processComplete;
        } else {
            throw new Exception("ProcessComplete must be in range 0-1");
        }
    }

    public String getName() {
        return path.substring(path.lastIndexOf("\\") + 1);
    }

    public void setProcessComplete(Integer processComplete) throws Exception {
        if (processComplete == 0 || processComplete == 1) {
            this.processComplete = processComplete;
        } else {
            throw new Exception("ProcessComplete must be in range 0-1");
        }
    }

    public void addCategory(String category) {
        this.categoriesSet.add(category);
    }

    public String getCategoriesSet() {
        StringBuilder stringBuilder = new StringBuilder();
        categoriesSet.forEach(s -> stringBuilder.append(s).append("|"));

        return stringBuilder.toString();
    }

    public Set<String> getCategorySet() {
        return categoriesSet;
    }

    public void setCategoriesSet(String categoriesSet) {
        this.categoriesSet = new HashSet<>(Arrays.asList(categoriesSet.split("\\|")));
    }

    public void setCategories(String categories) {
        this.categories = categories;
        setCategoriesSet(categories); // convertování strignu do SET pro lepší práci
    }

    public void setConcept(String concept) {
        this.concept = concept;
        setConceptSet(concept); // convertování strignu do SET pro lepší práci
    }

    public void addConcept(String concept) {
        this.conceptSet.add(concept);
    }

    public String getConceptSet() {
        StringBuilder stringBuilder = new StringBuilder();
        conceptSet.forEach(s -> stringBuilder.append(s).append("|"));

        return stringBuilder.toString();
    }

    public Set<String> getConceptSetSet() {
        return conceptSet;
    }

    public String getConcept() {
        return concept;
    }

    public void setConceptSet(String conceptSet) {
        this.conceptSet = new HashSet<>(Arrays.asList(conceptSet.split("\\|")));
    }

    public void setKeyword(String keyword) {
        this.keyword = keyword;
        setKeywordSet(keyword); // convertování strignu do SET pro lepší práci
    }

    public void addKeyword(String keyword) {
        this.keywordSet.add(keyword);
    }

    public String getKeywordSet() {
        StringBuilder stringBuilder = new StringBuilder();
        keywordSet.forEach(s -> stringBuilder.append(s).append("|"));

        return stringBuilder.toString();
    }

    public Set<String> getKeywordSetSet() {
        return keywordSet;
    }

    public void setKeywordSet(String keywordSet) {
        this.keywordSet = new HashSet<>(Arrays.asList(keywordSet.split("\\|")));
    }

    public Double getSentiment() {
        return sentiment;
    }

    public void setSentiment(double sentiment) {
        this.sentiment = sentiment;
    }

    public String getEmotionsString() {
        StringBuilder stringBuilder = new StringBuilder();
        for (Emotion emotion : getEmotions()) {
//            stringBuilder.append(emotion.toString()).append(System.lineSeparator());
            stringBuilder.append(emotion.toString());
        }
        return stringBuilder.toString();
    }

    public String getConceptString() {
        setConceptSet(concept);
        StringBuilder stringBuilder = new StringBuilder();
        for (String concept : conceptSet) {
            stringBuilder.append(concept).append(System.lineSeparator());
        }
        return stringBuilder.toString();
    }

    public String getCategoriesString() {
        setCategoriesSet(categories);
        StringBuilder stringBuilder = new StringBuilder();
        for (String categori : categoriesSet) {
            stringBuilder.append(categori).append(System.lineSeparator());
        }
        return stringBuilder.toString();
    }

    public List<Emotion> getEmotionForFile() {
        ArrayList<Emotion> emotionsForFile = new ArrayList<>();
        emotions.forEach(emotion -> {
            if (emotion.entity == null) {
                emotionsForFile.add(emotion);
            }
        } );
        return emotionsForFile;
    }
}
