package uhk.salay.npl.demonpl.watson;

import com.ibm.cloud.sdk.core.service.security.IamOptions;
import com.ibm.watson.natural_language_understanding.v1.NaturalLanguageUnderstanding;
import com.ibm.watson.natural_language_understanding.v1.model.*;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.StringReader;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.PostConstruct;
import javax.transaction.Transactional;

import org.apache.commons.io.FileUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.LazyInitializationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Service;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.process.CoreLabelTokenFactory;
import edu.stanford.nlp.process.PTBTokenizer;
import lombok.Setter;
import uhk.salay.npl.demonpl.jpa.model.Emotion;
import uhk.salay.npl.demonpl.jpa.model.Entity;
import uhk.salay.npl.demonpl.jpa.model.Part;
import uhk.salay.npl.demonpl.jpa.model.Relation;
import uhk.salay.npl.demonpl.jpa.model.Sentiment;
import uhk.salay.npl.demonpl.jpa.model.Type;
import uhk.salay.npl.demonpl.jpa.repositorys.EmotionsRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.EntityRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.FileRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.PartRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.RelationRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.SentimentRepository;
import uhk.salay.npl.demonpl.jpa.repositorys.TypeRepository;

@Setter
@Service
@PropertySource("classpath:project.properties")
public class WatsonService {

    private final static Logger logger = LogManager.getLogger(WatsonService.class);
    private NaturalLanguageUnderstanding naturalLanguageUnderstanding;
    @Value("${apiKey}")
    private String apiKey;
    @Value("${serviceUrl}")
    private String serviceUrl;
    @Value("${modelId}")
    private String modelId;
    @Value("${sizeOfPart:6000}")
    public int SIZE_OF_PART;
    @Value("${entity.emotion.anger}")
    private double emotionAnger;
    @Value("${entity.emotion.disgust}")
    private double emotionDisgust;
    @Value("${entity.emotion.fear}")
    private double emotionFear;
    @Value("${entity.emotion.joy}")
    private double emotionJoy;
    @Value("${entity.emotion.sadness}")
    private double emotionSadness;

    @Value("${file.emotion.anger}")
    private double fileAnger;
    @Value("${file.emotion.disgust}")
    private double fileDisgust;
    @Value("${file.emotion.fear}")
    private double fileFear;
    @Value("${file.emotion.joy}")
    private double fileJoy;
    @Value("${file.emotion.sadness}")
    private double fileSadness;


    private EntityRepository entityRepository;
    private RelationRepository relationsRepository;
    private TypeRepository typeRepository;
    private FileRepository fileRepository;
    private EmotionsRepository emotionsRepository;
    private SentimentRepository sentimentRepository;
    private PartRepository partRepository;

    public WatsonService() {
    }

    WatsonService(String apiKey, String serviceUrl) {
        this.apiKey = apiKey;
        this.serviceUrl = serviceUrl;
    }

    @Autowired
    public WatsonService(EntityRepository entityRepository, RelationRepository relationsRepository, @Qualifier("typeRepository") TypeRepository typeRepository, FileRepository fileRepository, EmotionsRepository emotionsRepository, SentimentRepository sentimentRepository, PartRepository partRepository) {
        this.entityRepository = entityRepository;
        this.relationsRepository = relationsRepository;
        this.typeRepository = typeRepository;
        this.fileRepository = fileRepository;
        this.emotionsRepository = emotionsRepository;
        this.sentimentRepository = sentimentRepository;
        this.partRepository = partRepository;
    }

    @PostConstruct
    public void init() throws FileNotFoundException {
        logger.info("init begin");
        naturalLanguageUnderstanding = new NaturalLanguageUnderstanding("2019-07-12",
                new IamOptions.Builder().apiKey(apiKey).build());
        naturalLanguageUnderstanding.setEndPoint(serviceUrl);
        logger.info("init end");
    }

    public Collection<String> getTokens(String textToAnalize) {
        PTBTokenizer<CoreLabel> ptbt = new PTBTokenizer<>(new StringReader(textToAnalize),
                new CoreLabelTokenFactory(), "");
        Collection<String> returnList = new ArrayList<>();
        while (ptbt.hasNext()) {
            CoreLabel label = ptbt.next();
            returnList.add(label.toString());
        }
        return returnList;
    }

    @Transactional
    public List<Entity> processTextOnEntities(AnalysisResults response, uhk.salay.npl.demonpl.jpa.model.File fileDb,
                                              Part part, boolean emotion, boolean sentiment, boolean mentions)
            throws Exception {
        List<Entity> collectionOfEntities = new ArrayList<>(response.getEntities().size());
        List<Sentiment> sentimentList = new ArrayList<>(response.getEntities().size());
        List<Emotion> emotionList = new ArrayList<>(response.getEntities().size());
        Set<String> collectionOfTypes = new HashSet<>();
        // vytvoření typů
        for (EntitiesResult entityResult : response.getEntities()) {
            collectionOfTypes.add(entityResult.getType());
        }
        List<Type> allTypes = typeRepository.findAll();
        collectionOfTypes.forEach(s -> {
            if (!typeExist(s, allTypes)) {
                final Type type = new Type();
                type.setName(s);
                typeRepository.saveAndFlush(type);
                allTypes.add(type);
            }
        });

        List<Entity> allEntities = entityRepository.findAll();
        for (EntitiesResult entityResult : response.getEntities()) {
            final Entity entity;

            if (!entityExist(allEntities, entityResult.getText())) {
                // vytvoř takovou
                entity = new Entity();
                entity.setName(entityResult.getText());
                entity.setType(getTypeFor(allTypes, entityResult.getType()));
                if (fileDb != null) {
                    entity.setFiles(new HashSet<uhk.salay.npl.demonpl.jpa.model.File>() {{
                        add(fileDb);
                    }});
                }
            } else {
                //  použi již existující
                entity = findInEntities(allEntities, entityResult.getText());
                try {
                    if (entity.getFiles() == null) {
                        entity.setFiles(new HashSet<>());
                    }
                    entity.getFiles().add(fileDb); // přidej nový file ve kterém se nachází
                } catch (LazyInitializationException e) {
                    entity.setFiles(new HashSet<>());
                    entity.getFiles().add(fileDb); // přidej nový file ve kterém se nachází

                }
            }
            entityRepository.save(entity);
            if (emotion) {
                final EmotionScores emotionScores = entityResult.getEmotion();
                if (emotionScores.getAnger() >= emotionAnger) {
                    createEmotion("Anger", emotionScores.getAnger(), entity, fileDb);
                }
                if (emotionScores.getDisgust() >= emotionDisgust) {
                    createEmotion("Disgust", emotionScores.getDisgust(), entity, fileDb);
                }
                if (emotionScores.getFear() >= emotionFear) {
                    createEmotion("Fear", emotionScores.getFear(), entity, fileDb);
                }
                if (emotionScores.getJoy() >= emotionJoy) {
                    createEmotion("Joy", emotionScores.getJoy(), entity, fileDb);
                }
                if (emotionScores.getSadness() >= emotionSadness) {
                    createEmotion("Sadness", emotionScores.getSadness(), entity, fileDb);
                }
            }
            if (sentiment) {
                final FeatureSentimentResults sentimentResults = entityResult.getSentiment();
                sentimentList.add(createSentiment(sentimentResults.getScore(), entity, fileDb));
            }
            collectionOfEntities.add(entity);
        }
        collectionOfEntities.forEach(entity -> entityRepository.saveAndFlush(entity));
        return collectionOfEntities;
    }


    @Transactional
    public List<Relation> processTextOnRelations(AnalysisResults response, uhk.salay.npl.demonpl.jpa.model.File fileDb) throws Exception {
        List<Relation> returnCollection = new ArrayList<>(response.getRelations().size());
        final List<Entity> entities = entityRepository.findAll();
        for (RelationsResult relation : response.getRelations()) {
            Entity fromEntity = findEntityOrCreate(relation.getArguments().get(0).getEntities().get(0), entities);
            Entity toEntity = findEntityOrCreate(relation.getArguments().get(1).getEntities().get(0), entities);

            if (toEntity == null) {
                final Entity entity = new Entity();
                entity.setName(relation.getArguments().get(1).getEntities().get(0).getText());
                Type type = typeRepository.findByName(relation.getArguments().get(1).getEntities().get(0).getType());
                if (type == null) {
                    Type type1 = new Type();
                    type1.setName(relation.getArguments().get(0).getEntities().get(1).getType());
                    typeRepository.saveAndFlush(type1);
                    type = type1;
                }
                toEntity.setType(type);
                toEntity = entity;
            }


            returnCollection.add(new Relation(relation.getType(),
                    fromEntity, // entityFromName
                    toEntity,  // entityToName
                    fileDb
            ));
        }
        final List<Relation> all = relationsRepository.findAll();
        for (Relation relation : returnCollection) {
            if (!all.contains(relation)) {
                relationsRepository.saveAndFlush(relation);
            }
        }
        return returnCollection;
    }

    public void loadDataFrom(String pathToLoad, Set<WatsonFeatures> watsonFeatures) throws Exception {
        logger.info("Data load start");
        File file = FileUtils.getFile(pathToLoad);
        if (file == null) {
            logger.error("FileNotFound");
            throw new FileNotFoundException("Path for file " + pathToLoad);
        }
        final String contentOfFile = FileUtils.readFileToString(file, StandardCharsets.UTF_8);
        final uhk.salay.npl.demonpl.jpa.model.File fileDb = saveFileToDb(file);
        List<Part> parts = cutFileToParts(fileDb, contentOfFile);
        logger.info(String.format("There is %s parts", parts.size()));
        if (modelId == null || modelId.isEmpty()) {
            modelId = null;
        }
        int partNumber = 0;
        Instant start = Instant.now();
        for (Part part : parts) {
            Instant startPart = Instant.now();
            logger.info("PartNumber " + ++partNumber);
            final Features.Builder featureBuilder = new Features.Builder();

            if (watsonFeatures.contains(WatsonFeatures.ENTITIES)) {
                EntitiesOptions.Builder entities = new EntitiesOptions.Builder().model(modelId);
                if (watsonFeatures.contains(WatsonFeatures.ENTITIES_EMOTION)) {
                    entities.emotion(true);
                }
                if (watsonFeatures.contains(WatsonFeatures.ENTITIES_SENTIMENT)) {
                    entities.sentiment(true);
                }
                if (watsonFeatures.contains(WatsonFeatures.ENTITIES_MENTIONS)) {
                    entities.mentions(true);
                }
                featureBuilder.entities(entities.build());
            }
            if (watsonFeatures.contains(WatsonFeatures.RELATIONS)) {
                RelationsOptions relations = new RelationsOptions.Builder().model(modelId).build();
                featureBuilder.relations(relations);
            }
            if (watsonFeatures.contains(WatsonFeatures.KEY_WORDS)) {
                KeywordsOptions keywordsOptions = new KeywordsOptions.Builder()
                        .sentiment(true)
                        .emotion(true)
                        .build();
                featureBuilder.keywords(keywordsOptions);
            }
            if (watsonFeatures.contains(WatsonFeatures.CONCEPTS)) {
                ConceptsOptions concepts = new ConceptsOptions.Builder()
                        .build();
                featureBuilder.concepts(concepts);
            }
            if (watsonFeatures.contains(WatsonFeatures.CATEGORIES)) {
                final CategoriesOptions categories = new CategoriesOptions.Builder()
                        .explanation(true)
                        .build();
                featureBuilder.categories(categories);
            }
            if (watsonFeatures.contains(WatsonFeatures.SENTIMENT)) {
                final SentimentOptions sentiment = new SentimentOptions.Builder()
                        .build();
                featureBuilder.sentiment(sentiment);
            }

            if (watsonFeatures.contains(WatsonFeatures.EMOTION)) {
                final EmotionOptions emotion = new EmotionOptions.Builder().build();
                featureBuilder.emotion(emotion);
            }
            // TODO: 04.01.2020 dodělat další features

            AnalyzeOptions parameters = new AnalyzeOptions.Builder()
//                    .text(contentOfFile)
                    .text(part.getText())
                    .features(featureBuilder.build())
                    .build();

            AnalysisResults response = naturalLanguageUnderstanding
                    .analyze(parameters)
                    .execute()
                    .getResult();

            if (watsonFeatures.contains(WatsonFeatures.ENTITIES)) {
                final List<Entity> entities = processTextOnEntities(response, fileDb, part,
                        watsonFeatures.contains(WatsonFeatures.ENTITIES_EMOTION),
                        watsonFeatures.contains(WatsonFeatures.ENTITIES_SENTIMENT),
                        watsonFeatures.contains(WatsonFeatures.ENTITIES_MENTIONS));
            }
            logger.info("ENTITIES done");
            if (watsonFeatures.contains(WatsonFeatures.RELATIONS)) {
                final List<Relation> relations = processTextOnRelations(response, fileDb);
            }
            logger.info("RELATIONS done");

            // zpacování daných feature a přidání k hlavnímu file
            if (watsonFeatures.contains(WatsonFeatures.KEY_WORDS)) {
                processTextOnKeyWords(response, part, fileDb);
            }
            logger.info("KEY_WORDS done");

            if (watsonFeatures.contains(WatsonFeatures.CATEGORIES)) {
                processCategories(response, part, fileDb);
            }
            logger.info("CATEGORIES done");

            if (watsonFeatures.contains(WatsonFeatures.CONCEPTS)) {
                processConcepts(response, part, fileDb);
            }
            logger.info("CONCEPTS done");
            if (watsonFeatures.contains(WatsonFeatures.SENTIMENT)) {
                part.setSentiment(response.getSentiment().getDocument().getScore());
                partRepository.saveAndFlush(part);
            }
            logger.info("SENTIMENT done");
            if (watsonFeatures.contains(WatsonFeatures.EMOTION)) {
                final EmotionScores emotionScores = response.getEmotion().getDocument().getEmotion();
                if (emotionScores.getAnger() >= fileAnger) {
                    createEmotion("Anger", emotionScores.getAnger(), null, fileDb);
                }
                if (emotionScores.getDisgust() >= fileDisgust) {
                    createEmotion("Disgust", emotionScores.getDisgust(), null, fileDb);
                }
                if (emotionScores.getFear() >= fileFear) {
                    createEmotion("Fear", emotionScores.getFear(), null, fileDb);
                }
                if (emotionScores.getJoy() >= fileJoy) {
                    createEmotion("Joy", emotionScores.getJoy(), null, fileDb);
                }
                if (emotionScores.getSadness() >= fileSadness) {
                    createEmotion("Sadness", emotionScores.getSadness(), null, fileDb);
                }
                fileRepository.saveAndFlush(fileDb);
            }
            logger.info("EMOTION done");
            logger.info(String.format("PartNumber %s in %s", partNumber, Duration.between(startPart, Instant.now())));
        }
        calculateSentimentForFile(parts, fileDb);
        fileDb.setProcessComplete(1);
        fileRepository.saveAndFlush(fileDb); // uložení že soubor byl zpracován
        logger.info(String.format("Duration %s in %s", partNumber, Duration.between(start, Instant.now())));
        logger.info("Data load end");
    }

    private void calculateSentimentForFile(List<Part> parts, uhk.salay.npl.demonpl.jpa.model.File fileDb) {
        Double sumOfSentiment = 0.0;
        for (Part part : parts) {
            sumOfSentiment += part.getSentiment();
        }
        double avg = sumOfSentiment / parts.size();
        fileDb.setSentiment(avg);
        fileRepository.saveAndFlush(fileDb);
    }

    public Collection<Entity> getEntities(String text) {
        EntitiesOptions entities = new EntitiesOptions.Builder().model(modelId)
                .sentiment(true)
                .build();

        Features features = new Features.Builder()
                .entities(entities)
                .build();

        AnalyzeOptions parameters = new AnalyzeOptions.Builder()
                .text(text)
                .features(features)
                .build();

        AnalysisResults response = naturalLanguageUnderstanding
                .analyze(parameters)
                .execute()
                .getResult();

        List<Entity> entitiesList = new ArrayList<>(response.getEntities().size());
        for (EntitiesResult entity : response.getEntities()) {
            entitiesList.add(new Entity(entity.getText(), new Type(entity.getType())));
        }
        return entitiesList;
    }

    public Collection<Entity> getEntitiesLocal(String text) {
        return entityRepository.findByNameLike(text + "%");
    }


    Sentiment createSentiment(Double score, Entity entity, uhk.salay.npl.demonpl.jpa.model.File fileDb) {
        final Sentiment sentiment = new Sentiment();
        sentiment.setScore(score);
        sentiment.setEntity(entity);
        sentiment.setFile(fileDb);
        sentimentRepository.saveAndFlush(sentiment);
        return sentiment;
    }

    void createEmotion(String emotionText, Double value, Entity entity, uhk.salay.npl.demonpl.jpa.model.File fileDb) {
        final Emotion emotion = new Emotion();
        emotion.setType(emotionText);
        emotion.setValue(value);
        emotion.setEntity(entity);
        emotion.setFile(fileDb);
        emotionsRepository.saveAndFlush(emotion);
    }

    uhk.salay.npl.demonpl.jpa.model.File saveFileToDb(File file) throws Exception {
        final uhk.salay.npl.demonpl.jpa.model.File fileDb = new uhk.salay.npl.demonpl.jpa.model.File(file.getAbsolutePath(), new HashSet<>(), new Integer(0));
        fileRepository.saveAndFlush(fileDb);
        return fileDb;
    }

    List<Relation> getRelationsFor(String entity, String type) {
        final Type typeObject = typeRepository.findByName(type);
        final List<Entity> entityObject = entityRepository.findByNameAndType(entity, typeObject);
        List<Relation> relations = new ArrayList<>();
        for (Entity entity1 : entityObject) {
            relations.addAll(relationsRepository.findRelationByEntityFrom(entity1));
            relations.addAll(relationsRepository.findRelationByEntityTo(entity1));
        }

        return relations;
    }

    @Transactional
    Entity findEntityOrCreate(RelationEntity relationEntity, List<Entity> entities) {
        Entity fromEntity = findInEntities(entities, relationEntity.getText());
        if (fromEntity == null) {
            final Entity entity = new Entity();
            entity.setName(relationEntity.getText());
            Type type = typeRepository.findByName(relationEntity.getType());
            if (type == null) {
                Type type1 = new Type();
                type1.setName(relationEntity.getType());
                typeRepository.saveAndFlush(type1);
                type = type1;
            }
            entity.setType(type);
            entityRepository.saveAndFlush(entity);
            fromEntity = entity;
        }
        return fromEntity;
    }

    NaturalLanguageUnderstanding getNaturalLanguageUnderstanding() {
        return naturalLanguageUnderstanding;
    }

    /**
     * Rozřeže daný soubor na jednotlívé části, kterou jsou lépe zpracovatelné pro NLP.
     *
     * @param fileDb        objet představující sobuor v Db.
     * @param contentOfFile obsah daného souboru v DB.
     * @return List, jednotlivých částí, které jsou již uloženy v DB.
     */
    List<Part> cutFileToParts(uhk.salay.npl.demonpl.jpa.model.File fileDb, String contentOfFile) {
        List<Part> parts = new ArrayList<>();
        while (contentOfFile.length() > SIZE_OF_PART) {
            final int endIndex = contentOfFile.substring(0, SIZE_OF_PART).lastIndexOf('.');
            final String textForPart = contentOfFile.substring(0, endIndex);
            final Part part = new Part();
            part.setText(textForPart);
            parts.add(part);
            part.setFileId(fileDb.getId());
            partRepository.saveAndFlush(part);
            fileDb.getParts().add(part);
            contentOfFile = contentOfFile.substring(endIndex);
        }
        if (contentOfFile.length() > 0) {
            final Part part = new Part();
            part.setText(contentOfFile);
            partRepository.saveAndFlush(part);
            parts.add(part);
            part.setFileId(fileDb.getId());
            fileDb.getParts().add(part);
        }
        fileDb.setPart(parts.size());
        fileRepository.saveAndFlush(fileDb);
        return parts;
    }

    void processTextOnKeyWords(AnalysisResults response, Part part, uhk.salay.npl.demonpl.jpa.model.File fileDb) {
        final List<KeywordsResult> keywords = response.getKeywords();
        StringBuilder stringBuilder = new StringBuilder();
        keywords.forEach(keywordsResult -> {
            fileDb.addKeyword(keywordsResult.getText());
            stringBuilder.append(keywordsResult.getText()).append("|");
        });
        fileDb.setKeyword(fileDb.getKeywordSet());
        part.setKeyword(fileDb.getKeywordSet());
        partRepository.saveAndFlush(part);
    }

    void processConcepts(AnalysisResults response, Part part, uhk.salay.npl.demonpl.jpa.model.File fileDb) {
        final List<ConceptsResult> concepts = response.getConcepts();
        StringBuilder stringBuilder = new StringBuilder();
        concepts.forEach(conceptsResult -> {
            fileDb.addConcept(conceptsResult.getText());
            stringBuilder.append(conceptsResult.getText()).append("|");
        });
        part.setConcept(stringBuilder.toString());
        fileDb.setConcept(fileDb.getConceptSet());
        partRepository.saveAndFlush(part);
    }

    void processCategories(AnalysisResults response, Part part, uhk.salay.npl.demonpl.jpa.model.File fileDb) {
        System.out.println();
        final List<CategoriesResult> categories = response.getCategories();
        StringBuilder stringBuilder = new StringBuilder();
        categories.forEach(categoriesResult -> {
                    fileDb.addCategory(categoriesResult.getLabel());
                    stringBuilder.append(categoriesResult.getLabel()).append("|");
                }
        );
        part.setCategory(stringBuilder.toString());
        fileDb.setCategories(fileDb.getCategoriesSet());
        partRepository.saveAndFlush(part);
    }

    private Entity findInEntities(List<Entity> entities, String name) {
        for (Entity entity : entities) {
            if (entity.getName().equals(name)) {
                return entity;
            }
        }
        return null;
    }

    private Type findInTypes(List<Type> entities, String name) throws Exception {
        for (Type type : entities) {
            if (type.getName().equals(name)) {
                return type;
            }
        }
        throw new Exception(String.format("For that name: %s dont exist Type", name));
    }

    private boolean entityExist(List<Entity> allEntities, String name) {
        for (Entity entity : allEntities) {
            if (name.equals(entity.getName())) {
                return true;
            }
        }
        return false;
    }

    private boolean typeExist(String s, List<Type> allTypesForNewType) {
        for (Type type : allTypesForNewType) {
            if (s.equals(type.getName())) {
                return true;
            }
        }
        return false;
    }

    private Type getTypeFor(List<Type> allTypes, String typeName) throws Exception {
        final List<Type> collect = allTypes.stream().filter(type -> type.getName().equals(typeName)).collect(Collectors.toList());
        Type typeForReturn = collect.size() == 1 ? collect.get(0) : null;
        if (typeForReturn == null) {
            throw new Exception("No or more types for > " + typeName);
        }
        return typeForReturn;
    }

    /**
     * Metoda vrati nazvy souboru v kterych se kontext nachazi. Pokud je prepinac justFile = True.
     * Metoda vrati nazvy souboru a presne parts ve kterych se kontext nachazi. pokud je prepinac justFile na = False.
     *
     * @param contentText text ktery se ma v contextu nachazet.
     * @param justFile    prepinac, jesli se maji vratit i presne parts.
     * @return seznam jmen souboru v kterych se concept nachazi. Pokud nenalezne nic vraci NULL.
     */
    public Set<String> getFileByContext(String contentText, boolean justFile) {
        Set<String> fileNames = null;
        if (justFile) {
            List<uhk.salay.npl.demonpl.jpa.model.File> byConceptLike = fileRepository.findByConceptLike("%" + contentText + "%");
            if (byConceptLike != null) {
                fileNames = new HashSet<>(byConceptLike.size());
                for (uhk.salay.npl.demonpl.jpa.model.File file : byConceptLike) {
                    fileNames.add(file.getPath().substring(file.getPath().lastIndexOf("/") + 1));
                }
            }
        } else {
            List<Part> byConceptLike = partRepository.findByConceptLike("%" + contentText + "%");
            if (byConceptLike != null) {
                fileNames = new HashSet<>(byConceptLike.size());
                for (uhk.salay.npl.demonpl.jpa.model.Part part : byConceptLike) {
                    Optional<uhk.salay.npl.demonpl.jpa.model.File> file = fileRepository.findById(part.getFileId());
                    if (file.isPresent()) {
                        fileNames.add(file.get().getPath().substring(file.get().getPath().lastIndexOf("/") + 1)
                                + " part: " + part.getId());
                    }
                }
            }
        }
        return fileNames;
    }

    public List<uhk.salay.npl.demonpl.jpa.model.File> getFileByPath(String nameOfBook) {
        return fileRepository.findByPathLike("%" + nameOfBook + "%");
    }

    public List<uhk.salay.npl.demonpl.jpa.model.File> getAllFile() {
        return fileRepository.findAll();
    }
}
